@@ -61,10 +61,15 @@ read.dta <- function(file, convert.dates = TRUE,
     }
 
     if (convert.dates) {
-        ff <- attr(rval,"formats")
-        dates <- grep("%-*d", ff)
-        ## avoid as.Date in case strptime is screwed up
-        base <- structure(-3653, class="Date")
+        ff <- attr(rval, "formats")
+        ## dates <- grep("%-*d", ff)
+        ## Stata 12 introduced 'business dates'
+        ## 'Formats beginning with %t or %-t are Stata's date and time formats.'
+        ## but it seems some are earlier.
+        dates <- if (attr(rval, "version") >= 8L) grep('%-*d|^%t|%-t', ff)
+        else grep("%-*d", ff)
+        ## avoid as.Date in case strptime is messed up
+        base <- structure(-3653, class = "Date")
         for(v in dates) rval[[v]] <- base+rval[[v]]
     }
     if (convert.factors %in% c(TRUE, NA)) {
@@ -78,7 +83,7 @@ read.dta <- function(file, convert.dates = TRUE,
                 labels <- tt[[ll[v]]]
                 if (warn.missing.labels && is.null(labels)) {
                     warning(gettextf("value labels (%s) for %s are missing",
-                                     ll[v], names(rval)[v]),
+                                     sQuote(ll[v]), sQuote(names(rval)[v])),
                             domain = NA)
                     next
                 }
@@ -160,9 +165,10 @@ write.dta <-
     shortlevels <- function(f) {
         ll <- levels(f)
         if (is.null(ll)) return(NULL)
-        abbreviate(ll, 80L)
+        ## avoid warning if non-ASCII strings are used (unwisely)
+        if (all(nchar(ll, "bytes") <= 80L)) ll else abbreviate(ll, 80L)
     }
-    leveltable <- lapply(dataframe,shortlevels)
+    leveltable <- lapply(dataframe, shortlevels)
 
     if (any(sapply(dataframe, function(x) {
         d <- dim(x)
